%Program for the one-dimensional Villemoes problem

%written in winter 1999 by N.Mulders
%modified by A.Barinka,
%IGPM, RWTH-Aachen, GERMANY
%for information, read READ_ME

%----------------------------------------
function doOne(mask, N1, N2, M, r, mode)
%----------------------------------------

%with:
%mask a={a_k}
%N1 is the lower bound, N2 the upper bound of the set Omega=[N1,N2]
%M is the 1-by-1 dilation matrix
%r is the order of the Strang-Fix conditions
%mode is the mode for the output of this program;
%   there are 3 modes for the output:
%   0=nothing
%   1=almost everything
%   2=only eigenvalues and eigenvectors

global r_regular;    %r_regular is the lower bound for the Sobolev regularity
global index_A_A;    %index_A_A is a variable for the output (in case the index
		     %k_k into vector A_A(k_k) is negative or zero)
global no_ev;        %no_ev is a variable for the output (in case none of
    		     %the eigenvectors is in the set V_r)
global spec_rad;     %spec_rad is a variable for the output (in case the
		     %spectral radius is zero)
global EW_M;         %EW_M is a variable for the output (in case the absolute
		     %value of the largest eigenvalue of M is 1) 
global interpol_str; %is the scaling function interpolating? (output) 
global time_str;     %time_str is the total time in minutes needed for the
                     %calculation of the lower bound (incl. examination of
		     %interpolation)

FIGURE=gcf;

format long;

klok=clock;

if (mode>=1)
  disp('   ')
  disp('                    >>>>>>>  Output of doOne  <<<<<<<')
  disp('   ')
end

if (N2<=N1)  
     do='***** WARNING: N2 is not larger than N1! Setting N2=N1+1. *****';
     if isempty(h_h)
        disp(do)
     else
        text_out(do);
        h_h=findobj('Tag','N2Box');
        N_N=num2str(N1+1);
        set(h_h,'String',N_N);
        pause(2);
        if (mode==1) disp(do); end;
     end;
     N2=N1+1;
end

N=N2-N1;
h=1;     %Stepsize


%---Determine the indices is of the matrix As
clear is
n=(N/h+1);
if (mode==1)
  disp('Order n of n-by-n matrix As:')
  n
end

ismin=N1;
ismax=N2;
for i=1:n
  for j=1:n
    is(i,j)=M*(i-1+N1)-(j-1+N1);
  end
end

ismin=min(min(is));
ismax=max(max(is));

if (mode==1)
  disp('Smallest index of the matrix-elements is:')
  disp(ismin)
  disp('Largest index of the matrix-elements is:')
  disp(ismax) 
end


%---Assemble vector A_A which consists of the elements of the mask 
m=ismax-ismin+1; %number of components of vector A_A
v=-ismin+1;      %necessary shift for the indices (to become positive) 

if (mode==1)   
  disp('Shift:');
  v
end

if (min(mask(:,1))+v<=0)
   index_A_A=0;
   do={'The interval [N1,N2] is too small for the chosen mask!' ...
       'Please enter new parameters N1 and N2.'};
   if isempty(h_h)
      disp(do)
   else
      text_out(do,[0.5 0 0]);
      pause(0.1);
      if (mode==1) disp(do); end;
   end;
  
else %intervall ok
  index_A_A=1; 
  s_s=size(mask);
  s_s=s_s(1);
  A_A=zeros(1,m);

  for i=1:s_s
    k_k=mask(i,1)+v;
    A_A(k_k)=mask(i,2);
  end


  %Assembly of matrix As
  As=zeros(n,n);
  for i=1:n
    for j=1:n
      new_is=is(i,j)+v;     %Shift of the indices so that they become positive
      As(i,j)=A_A(new_is);       
    end
  end   
  if (mode==1)   
    disp('Matrix As:');
    As
  end


  %---Computation and sorting of the eigenvalues and eigenvectors of matrix As
  [EV1,EW1]=eig(As);
  [W,P]=sort(eig(As));  
  EV=zeros(n,n);
  EW=zeros(n,n);
  for w=1:n
    EV(:,w)=EV1(:,P(w));           
    EW(:,w)=EW1(:,P(w));          
  end

  if (mode>=1)   
    disp('The eigenvalues of As are:')
    disp(W)
    disp('The eigenvectors of As are:')
    EV
  end


  %---Inverse transform of the indices k of the mask a={a_k}
  clear k
  for f=1:n
    k(f)=f-1+N1;
  end


  %---Check to each eigenvector of As, if it is contained in the set V_r
  epsilon=1.D-7;
  pos=0;
  kein=0;
  clear Nr_EV  
  Nr_EV=0;
  clear summe       
  for m=1:r+1              %r+1 is the number of the Strang-Fix conditions 
    summe(m,:)=k.^(m-1)*EV;    
  end   
             
  for j=1:n     
    if (sum(abs(summe(:,j)))<epsilon)
      pos=pos+1;
      Nr_EV(pos)=j;       %Number of the eigenvector that is an element of V_r
    else
      kein=kein+1;
    end
  end


  %---Check to multiple eigenvalues, if the linear combination of the 
  %corresponding eigenvectors is contained in the set V_r
  g=1;
  num=0;
  clear Nr_EW
  Nr_EW=0;
  while g<=n 
    index=find(Nr_EV==g);
    row=find(abs(W-W(g))<epsilon);
    s=size(row,1);
    if (s>=2)&(isempty(index))
      clear LK
      LK=zeros(r+1,s);     
      for ilk=1:r+1
        for jlk=1:s
          LK(ilk,jlk)=summe(ilk,row(jlk));
        end
      end
      LK;
      if rank(LK,epsilon)<s
        num=num+1;
        Nr_EW(num)=g;     %Number of the corresponding multiple eigenvalue  
      end         
    end
    g=g+s;
  end


  if (kein==n)&(num==0)
    no_ev=0;
    if (mode>=1)
      disp('None of the eigenvectors is in the set V_r !!!')
      disp('   ') 
    end
  else
    no_ev=1;
    if (mode==1)
      disp('Following eigenvectors of the matrix As are in the set V_r:')
      disp('Number eigenvector:')
      disp(Nr_EV)
      disp('The linear combination of eigenvectors to following multiple eigenvalues')
      disp('are in V_r:')
      disp('Number eigenvalue:')
      disp(Nr_EW)
    end

    %---Determine the spectral radius of As restricted to V_r
    clear Nr
    clear Betrag_EW
    if Nr_EV==0
      Nr=[Nr_EW];
    else
      Nr=[Nr_EV,Nr_EW];
    end
    for t=1:pos+num
      Betrag_EW(t)=abs(W(Nr(t)));
    end

   Max_EW=max(Betrag_EW);
    if (mode>=1)
      disp('The spectral radius of As restricted to V_r is:')
      disp(Max_EW)
    end

    %---The lower bound for the Sobolev regularity
    if (Max_EW==0)
      spec_rad=0;
      if (mode>=1)
        disp('The spectral radius is zero, and therefore the estimated lower bound') 
        disp('for the Sobolev regularity would become infinity!')
        disp('   ')
      end
    else
      spec_rad=1;
      Max_EW_M=abs(max(eig(M)));
      if (Max_EW_M==1)
        EW_M=0;
        if (mode>=1)
          disp('Warning: Division by zero! (The absolute value of the largest')
          disp('eigenvalue of the dilation matrix M is 1!)')
          disp('   ') 
        end
      else
        EW_M=1;  
        Kappa_U=-log(Max_EW)/log(Max_EW_M);
        r_regular=Kappa_U;
        if (mode>=1)
          disp('Estimated lower bound for the Sobolev regularity is:')
          disp(Kappa_U)
        end
      end
    end 
    if (mode>=1)
      disp('><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><')
      disp('   ')
    end 
  end


  %---Examine whether the function is interpolating

  MaxEW_As=max(W);
  if (abs(MaxEW_As-1)<epsilon)
    nr=find(abs(W-MaxEW_As)<epsilon);
    snr=size(nr,1);
    if (snr>1)
       interpol_str='Function is not interpolating. (1 is a multiple eigenvalue.)';
    elseif (snr==1)
       comp_1=(n-1)/2 + 1;
       if (abs(EV(comp_1,nr)-1)<epsilon)
          %   comp_0=find(abs(EV(:,nr)-0)<epsilon);
          %   scomp_0=size(comp_0,1);
          %   if (scomp_0==n-1)   
          vec=EV(:,nr);
          vec(comp_1)=0;
          if (norm(vec)<epsilon)
             interpol_str='Function is interpolating !';
          else
             interpol_str='Function is not interpolating.';
          end
       else
          interpol_str='Function is not interpolating.';
       end
    end
  else
    interpol_str='Function is not interpolating.';
  end

time=etime(clock,klok);
time_str=num2str(time/60);

end %intervall ok



%----------------------
function text_out(out_text,out_color, out_value);
%----------------------
if (nargin<=2) out_value=1; end;
if (nargin<=1) out_color=[0 0 0.5]; end;
if (nargin==0) out_text=''; end;

hh=findobj(FIGURE,'Tag','TextBox');
set(hh,'ForegroundColor',out_color);
set(hh,'String', out_text);
set(hh,'Value',out_value);












