%Program for the two-dimensional problem
%(Computes all eigenvectors and eigenvalues of the matrix As)

%written in autumn 1999 by N.Mulders, 
%modified by A.Barinka,
%IGPM, RWTH-Aachen, GERMANY
%for information, read READ_ME

%----------------------------------------
function doTwo(mask, N1, N2, M, r, mode)
%----------------------------------------

%with:
%mask a={a_k}
%N1 is the lower bound, N2 the upper bound of the set Omega=[N1,N2]x[N1,N2]
%M is the 2-by-2 dilation matrix
%r is the order of the Strang-Fix conditions
%mode is the mode for the output of this program;
%there are 3 modes for the output:
%0=nothing
%1=almost everything
%2=eigenvalues and eigenvectors

global r_regular;    %r_regular is the lower bound for the Sobolev regularity
global index_A_A;    %index_A_A is a variable for the output (in case the index
		     %k_k into matrix A_A(k_k) is negative or zero) 
global no_ev;        %no_ev is a variable for the output (in case none of
		     %the eigenvectors is in the set V_r)
global spec_rad;     %spec_rad is a variable for the output (in case the
		     %spectral radius is zero)  
global EW_M;         %EW_M is a variable for the output (in case the absolute
		     %value of the largest eigenvalue of M is 1)
global interpol_str; %is the scaling function interpolating?  (output)  
global interpol_num; %interpol_num is a variable needed for the visual program
                     %VisTwo.m:
                     %interpol_num=0 in case the function is not interpolating,
                     %interpol_num=1 in case the function is interpolating 
global big_ev;       %big_ev is the eigenvector to the largest eigenvalue of 
                     %the matrix As 
global time_str;     %time_str is the total time in minutes needed for the 
                     %calculation of the lower bound (incl. examination of
		     %interpolation)

FIGURE=gcf;

format long;

klok=clock;

if (mode>=1)
  disp('      ')
  disp('                    >>>>>>>  Output of doTwo  <<<<<<<')
  disp('      ')
end

if (N2<=N1)  
    
     do='***** WARNING: N2 is not larger than N1! Setting N2=N1+1. *****';
     if isempty(h_h)
        disp(do)
     else
        text_out(do);
        h_h=findobj('Tag','N2Box');
        N_N=num2str(N1+1);
        set(h_h,'String',N_N);
        pause(2);
        if (mode==1) disp(do); end;
     end;
     N2=N1+1;
end

N=N2-N1; 
h=1;     %Stepsize


%---Determine the indices is and js of the matrix As

clear is
clear js
n=(N/h+1)^2;
if (mode==1)
  disp('Order n of n-by-n matrix As:')
  n
end

ismin=N1;
ismax=N2;
jsmin=N1;
jsmax=N2;

for i=1:n
  for j=1:n
    is(i,j)=M(1,1)*(invF1(i,N)+N1)+M(1,2)*(invF2(i,N)+N1)-(invF1(j,N)+N1);
    js(i,j)=M(2,1)*(invF1(i,N)+N1)+M(2,2)*(invF2(i,N)+N1)-(invF2(j,N)+N1);
  end
end

ismin=min(min(is));
ismax=max(max(is));
jsmin=min(min(js));
jsmax=max(max(js));

smin=min(ismin,jsmin);
smax=max(ismax,jsmax);

if (mode==1) 
  disp('Smallest index of the matrix-elements is:') 
  disp(smin)
  disp('Largest index of the matrix-elements is:')
  disp(smax)
end 


%---Assemble m-by-m matrix A_A which consists of the elements of the mask 
m=smax-smin+1; %order m of m-by-m matrix A_A 
v=-smin+1;     %necessary shift for the indices (to become positive)

if (mode==1)
  disp('Shift:')
  v
end

if (min(mask(:,1))+v<=0)
  index_A_A=0;
     do={'The interval [N1,N2] is too small for the chosen mask!' ...
       'Please enter new parameters N1 and N2.'};
   if isempty(h_h)
      disp(do)
   else
      text_out(do,[0.5 0 0]);
      pause(0.1);
     if (mode==1) disp(do); end;
   end; 

else %intervall ok
  index_A_A=1;
  s_s=size(mask);
  s_s=s_s(1);
  A_A=zeros(m,m);

  for i=1:s_s
     k_k=mask(i,1)+v;
     l_l=mask(i,2)+v;
     A_A(k_k, l_l)=mask(i,3);
  end


  %Assembly of matrix As

  As=zeros(n,n);
  for i=1:n
    for j=1:n
      new_is=is(i,j)+v;     %Shift of the indices so that they become positive
      new_js=js(i,j)+v;         
      As(i,j)=A_A(new_is,new_js);
    end
  end   
  if (mode==1)
    disp('Matrix As:')
    As
  end
 

  clear is;
  clear js;

  %---Computation and sorting of the eigenvalues and eigenvectors of matrix As

 [EV1,EW1]=eig(As);
 [W,P]=sort(diag(EW1));
 EV=zeros(n,n);
 EW=zeros(n,n);
 for w=1:n
   EV(:,w)=EV1(:,P(w));
   EW(:,w)=EW1(:,P(w));
 end

  clear As;

  if (mode>=1)
    disp('The eigenvalues of As are:')
    disp(W)
    disp('The eigenvectors of As are:')
    EV
  end


  %---Inverse transform of the indices k=(k1,k2) of the mask a={a_k} 
  clear k1
  clear k2
  for f=1:n
    k1(f)=invF1(f,N)+N1;
    k2(f)=invF2(f,N)+N1;
  end  
  
  big_ev=EV(:,n);


  %---Check to each eigenvector of As, if it is contained in the set V_r
  epsilon=1.D-6;
  pos=0;
  kein=0;
  sfb=1;
  clear Nr_EV
  Nr_EV=0;
  clear summe   
  summe(1,:)=sum(EV);
  for ord=1:r
    q=0;
    while ord-q>=0
      sfb=sfb+1;         %sfb is the number of the Strang-Fix conditions 
      summe(sfb,:)=(k1.^(ord-q).*k2.^q)*EV;
      q=q+1;
    end
  end
  

  for j=1:n            
    if (sum(abs(summe(:,j)))<epsilon)
      pos=pos+1;
      Nr_EV(pos)=j;      %Number of the eigenvector that is an element of V_r
    else
      kein=kein+1;
    end
  end
  
  %---Check to multiple eigenvalues, if the linear combination of the 
  %corresponding eigenvectors is contained in the set V_r

  g=1;
  num=0;
  clear Nr_EW
  Nr_EW=0;
  while g<=n
    index=find(Nr_EV==g);
    row=find(abs(W-W(g))<epsilon);
    s=size(row,1);
    if (s>=2)&(isempty(index))
      clear LK
      LK=zeros(sfb,s);     
      for ilk=1:sfb
        for jlk=1:s
          LK(ilk,jlk)=summe(ilk,row(jlk));
        end 
      end
      LK;
      if rank(LK,epsilon)<s
        num=num+1;
        Nr_EW(num)=g;      %Number of the corresponding multiple eigenvalue 
      end                    
    end
    g=g+s;
  end

  if (kein==n)&(num==0)
    no_ev=0;
    disp('None of the eigenvectors is in the set V_r!!!')
    disp('   ')
  else
    no_ev=1;
    if (mode==1)
      disp('Following eigenvectors of the matrix As are in the set V_r:')
      disp('Number eigenvector:')
      disp(Nr_EV) 
      disp('The linear combination of eigenvectors to following multiple eigenvalues')
      disp('are in V_r:')
      disp('Number eigenvalue:')
      disp(Nr_EW)
    end

    %---Determine the spectral radius of As restricted to V_r
    clear Nr
    clear Betrag_EW
    if Nr_EV==0
      Nr=[Nr_EW];
    else
      Nr=[Nr_EV,Nr_EW];
    end 
    for t=1:pos+num
      Betrag_EW(t)=abs(W(Nr(t)));
    end

    Max_EW=max(Betrag_EW);
    if (mode>=1)
      disp('The spectral radius of As restricted to V_r is:')
      disp(Max_EW)
    end 


    %---The lower bound for the Sobolev regularity
    if (Max_EW==0)
      spec_rad=0;
      if (mode>=1)
        disp('The spectral radius is zero, and therefore the estimated lower bound') 
        disp('for the Sobolev regularity would become infinity!')
        disp('   ')
      end
    else
      spec_rad=1;
      Max_EW_M=abs(max(eig(M)));
      if (Max_EW_M==1)
        EW_M=0;
        if (mode>=1)
          disp('Warning: Division by zero! (The absolute value of the largest')
          disp('eigenvalue of the dilation matrix M is 1!)')
          disp('   ')
        end 
      else
        EW_M=1;   
        Kappa_U=-log(Max_EW)/log(Max_EW_M);
        r_regular=Kappa_U;
        if (mode>=1)
          disp('Estimated lower bound for the Sobolev regularity is:')
          disp(Kappa_U)
        end
      end
    end
    if (mode>=1)
      disp('><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><')
      disp('   ')
    end
  end


  %---Examine whether the function is interpolating

  MaxEW_As=max(W);
  if (abs(MaxEW_As-1)<epsilon)
    nr=find(abs(W-MaxEW_As)<epsilon);
    snr=size(nr,1);
    if (snr>1)
      interpol_str='Function is not interpolating. (1 is a multiple eigenvalue.)';
      interpol_num=0;
    elseif (snr==1)
      comp_1=(n-1)/2 + 1;
      if (abs(EV(comp_1,nr)-1)<epsilon)
           vec=EV(:,nr);
        vec(comp_1)=0;
        if (norm(vec)<epsilon) 
          interpol_str='Function is interpolating !';
          interpol_num=1;
        else
          interpol_str='Function is not interpolating.';
          interpol_num=0;
        end
      else
         interpol_str='Function is not interpolating.';
         interpol_num=0;
      end
    end
  else
    interpol_str='Function is not interpolating.';
    interpol_num=0;
  end

time=etime(clock,klok);
time_str=num2str(time/60);
 
 if (interpol_num==0)
  save EV.mat big_ev;
  big_ev;
 end


end %intervall ok

%-------------------------------------------------------------------
%help routines

%-------------------------------
function F_invers1=invF1(A1,A2)
%-------------------------------
% invF1 computes the first component of the inverse of F
F_invers1=A1-(A2+1)*floor((A1-1)/(A2+1))-1;


%------------------------------
function F_invers2=invF2(A1,A2)
%------------------------------
% invF1 computes the second component of the inverse of F
F_invers2=floor((A1-1)/(A2+1));


%----------------------
function text_out(out_text,out_color, out_value);
%----------------------
if (nargin<=2) out_value=1; end;
if (nargin<=1) out_color=[0 0 0.5]; end;
if (nargin==0) out_text=''; end;

hh=findobj(FIGURE,'Tag','TextBox');
set(hh,'ForegroundColor',out_color);
set(hh,'String', out_text);
set(hh,'Value',out_value);



















