%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Filename:     gui_plot2d.m
% Written by:   Markus Hampel
% Version:      1.0, 2003
%
% Description:
% Multiwavelet plotting (Dimension d=2)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function varargout = gui_plot2d(varargin)
% GUI_PLOT2D Application M-file for gui_plot2d.fig
%    FIG = GUI_PLOT2D launch gui_plot2d GUI.
%    GUI_PLOT2D('callback_name', ...) invoke the named callback.

% Last Modified by GUIDE v2.0 06-Jul-2003 18:23:14

if nargin == 0  % LAUNCH GUI

	fig = openfig(mfilename,'reuse');

	% Use system color scheme for figure:
	set(fig,'Color',get(0,'defaultUicontrolBackgroundColor'));

	% Generate a structure of handles to pass to callbacks, and store it. 
	handles = guihandles(fig);
	guidata(fig, handles);
    
    %============================================================
    % setting some defaults
    %global m_mask;
    %set(handles.edit_N1, 'String', min(m_mask(:, 1)));
    %set(handles.edit_N2, 'String', max(m_mask(:, 1)));
    %============================================================

	if nargout > 0
		varargout{1} = fig;
	end

elseif ischar(varargin{1}) % INVOKE NAMED SUBFUNCTION OR CALLBACK

	try
		if (nargout)
			[varargout{1:nargout}] = feval(varargin{:}); % FEVAL switchyard
		else
			feval(varargin{:}); % FEVAL switchyard
		end
	catch
		disp(lasterr);
	end

end


% --------------------------------------------------------------------
function varargout = button_plot_Callback(h, eventdata, handles, varargin)

%global r;
r = 1;
%global A;
global m_mask;
global M_M;

M = M_M;

% Maskenformat konvertieren
[A, versch_L1, versch_L2] = m_mask2A(m_mask);
     
nIter = str2num(get(handles.edit_nIter, 'String'));
offset(1) = str2num(get(handles.edit_offsetX, 'String')) - versch_L1;
offset(2) = str2num(get(handles.edit_offsetY, 'String')) - versch_L2;
if get(handles.radiobutton_autoSupp, 'Value') == 1
    supp = 'a';
else
    L1 = str2num(get(handles.edit_L1, 'String')) + versch_L1;
    L2 = str2num(get(handles.edit_L2, 'String')) + versch_L2;
    R1 = str2num(get(handles.edit_R1, 'String')) + versch_L1;
    R2 = str2num(get(handles.edit_R2, 'String')) + versch_L2;
    supp = [L1 R1 R1 L1; ...
            L2 L2 R2 R2];
end
set(handles.text_calc, 'Visible', 'on');
% Fenster neu zeichnen
drawnow;

% Wavelet berechnen und Zeit messen
saving = get(handles.checkbox_saving, 'Value');
file = get(handles.edit_datafile, 'String');
if isempty(file)
    file = '';
end
tic
[Phi, errormsg] = plotScalingvector(A, M, nIter, supp, saving, file, offset);
toc

set(handles.text_calc, 'Visible', 'off');

% Auswahl-GUI loeschen
delete(handles.figure_plot2d);

% Wavelet zeichnen
if strcmp(errormsg, '') ~= 1
    disp(errormsg)
else
    for i = 3:3+r-1
        figure;
        % Gitter triangulieren:
        tri = delaunay(Phi(1,:), Phi(2,:));
        trisurf(tri, Phi(1,:), Phi(2,:), Phi(i,:))
        shading interp;
        lighting gouraud;
        %title(['\Phi_', num2str(i-2)]);
    end
end


% --------------------------------------------------------------------
function varargout = edit_L1_Callback(h, eventdata, handles, varargin)

set(handles.edit_L1a, 'String', get(handles.edit_L1, 'String')); 


% --------------------------------------------------------------------
function varargout = edit_L2_Callback(h, eventdata, handles, varargin)

set(handles.edit_L2a, 'String', get(handles.edit_L2, 'String')); 


% --------------------------------------------------------------------
function varargout = edit_R1_Callback(h, eventdata, handles, varargin)

set(handles.edit_R1a, 'String', get(handles.edit_R1, 'String')); 


% --------------------------------------------------------------------
function varargout = edit_R2_Callback(h, eventdata, handles, varargin)

set(handles.edit_R2a, 'String', get(handles.edit_R2, 'String')); 


% --------------------------------------------------------------------
function varargout = radiobutton_autoSupp_Callback(h, eventdata, handles, varargin)

set(handles.radiobutton_autoSupp, 'Value', 1);
set(handles.radiobutton_manSupp, 'Value', 0);
set(handles.text_L1, 'Enable', 'off');
set(handles.edit_L1, 'Enable', 'off');
set(handles.text_L2, 'Enable', 'off');
set(handles.edit_L2, 'Enable', 'off');
set(handles.text_R1, 'Enable', 'off');
set(handles.edit_R1, 'Enable', 'off');
set(handles.text_R2, 'Enable', 'off');
set(handles.edit_R2, 'Enable', 'off');
%set(handles.checkbox_saving, 'Enable', 'on');
%set(handles.text_datafile, 'Enable', 'on');
%set(handles.edit_datafile, 'Enable', 'on');
%set(handles.button_choose, 'Enable', 'on');


% --------------------------------------------------------------------
function varargout = radiobutton_manSupp_Callback(h, eventdata, handles, varargin)

set(handles.radiobutton_manSupp, 'Value', 1);
set(handles.radiobutton_autoSupp, 'Value', 0);
set(handles.text_L1, 'Enable', 'on');
set(handles.edit_L1, 'Enable', 'on');
set(handles.text_L2, 'Enable', 'on');
set(handles.edit_L2, 'Enable', 'on');
set(handles.text_R1, 'Enable', 'on');
set(handles.edit_R1, 'Enable', 'on');
set(handles.text_R2, 'Enable', 'on');
set(handles.edit_R2, 'Enable', 'on');
%set(handles.checkbox_saving, 'Enable', 'off');
%set(handles.text_datafile, 'Enable', 'off');
%set(handles.edit_datafile, 'Enable', 'off');
%set(handles.button_choose, 'Enable', 'off');


% --------------------------------------------------------------------
function varargout = button_choose_Callback(h, eventdata, handles, varargin)

cd ./Data/Plot
[filename, pathname] = uigetfile('*.mat', 'Pick an MAT-file');
cd ../..
if filename ~= 0 | pathname ~= 0
    set(handles.edit_datafile, 'String', [pathname, filename]);
    
    set(handles.radiobutton_manSupp, 'Value', 1);
    set(handles.radiobutton_autoSupp, 'Value', 0);
    set(handles.text_L1, 'Enable', 'on');
    set(handles.edit_L1, 'Enable', 'on');
    set(handles.text_L2, 'Enable', 'on');
    set(handles.edit_L2, 'Enable', 'on');
    set(handles.text_R1, 'Enable', 'on');
    set(handles.edit_R1, 'Enable', 'on');
    set(handles.text_R2, 'Enable', 'on');
    set(handles.edit_R2, 'Enable', 'on');
    load([pathname, filename], 'suppPhi');
    set(handles.edit_L1, 'String', num2str(suppPhi(1,1)));
    set(handles.edit_L2, 'String', num2str(suppPhi(2,1)));
    set(handles.edit_R1, 'String', num2str(suppPhi(1,3)));
    set(handles.edit_R2, 'String', num2str(suppPhi(2,3)));
    
    set(handles.edit_L1a, 'String', num2str(suppPhi(1,1)));
    set(handles.edit_L2a, 'String', num2str(suppPhi(2,1)));
    set(handles.edit_R1a, 'String', num2str(suppPhi(1,3)));
    set(handles.edit_R2a, 'String', num2str(suppPhi(2,3)));
end
