%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Filename:     plotScalingvector.m
% Version:      1.0, 2003
%
% Description:
% Plots scaling vectors with compact support.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Phi, errormsg] = plotScalingvector(A, M, nIter, manSupp, saving, datafile, offset)
% A        : Maske im internen Format
% M        : Dilatationsmatrix
% nIter    : Anzahl Iterationen
% xOffset  : Verschiebung des Graphen entlang der x-Achse
% manSupp  : gibt an, ob der Trger manuell bestimmt wurde
%            --> evtl. keine automatische Abschtzung!
% saving   : = 1 --> Lsung des EV-Problems speichern
% datafile : Name und Pfad der Datei mit gespeicherter Lsung
%            des EV-Problems

% Dimension
d = size(M, 1);
% Multiplizitt
r = size(A{1}, 1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dimension d = 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if d == 1
    L = 0;              % kleinster Index
    R = size(A, 1) - 1; % grsster Index (= N-1)
    % Dimension des Eigenwert/-vektor Problems ermitteln
    dimEV = R - L - 1;
    % Matrix zum EV-Problem aufstellen
    P = zeros(r*dimEV, r*dimEV);
    for i = 1:dimEV
        count = 0; % Index des Symbols
        argument = M*i - count;
        while argument > L
            if argument < R & count <= R
                column = L + argument;
                P((i-1)*r+1:(i-1)*r+r, (column-1)*r+1:(column-1)*r+r) = A{count + 1};
            end
            count = count + 1;
            argument = M*i - count;
        end
    end
    
    % Eigenvektoren V und Eigenwerte D bestimmen, es gilt: P*V = V*D
    [V, D]=eig(P);
    % Matrix P wird nicht mehr bentigt:
    clear('P');
    % Eigenwert 1 in D suchen
    eigOne = [0, 0];    % eigOne(1) = column, eigOne(2) = number of Eig=1
    for i = 1:size(D, 1)
        if D(i, i) == 1
            eigOne(1) = i;
            eigOne(2) = eigOne(2) + 1;
        end
    end
    if eigOne(2) == 0
        errormsg = 'Error: 1 not Eigenvalue of P';
        Phi = [];
        return
    elseif eigOne(2) > 1
        errormsg = 'Error: 1 not unique Eigenvalue of P';
        Phi = [];
        return
    else
        errormsg = '';
    end
    % Anzahl der zu berechnenden Punkte von Phi bestimmen
    % Berechnung auf ganzen Zahlen ist bereits ein Iterationsschritt
    nPoints = power(M, nIter-1) * (R - L) + 1;
    % berechnete Funktionswerte von Phi = (Phi_1, ..., Phi_r)
    Phi = zeros(r+1, nPoints);
    % 1. Zeile Argumente, Zeile 2,..,r+1 gehren zu Phi_1,..,Phi_r
    
    % Argumentenzeile initialisieren
    argument = L;
    for i = 1:nPoints
        Phi(1, i) = argument;
        argument = argument + 1/power(M, nIter-1);
    end
    % berechnete Werte fr ganze Zahlen eintragen
    % Phi auf den ganzen Zahlen ist gleich dem berechneten EV zum EW 1
    % Wissen aus Stetigkeit: Phi_j(L) = Phi_j(R) = 0
    %Phi_iter = V(:, eigOne(1));
    for arg = L+1:R-1
        Phi(2:r+1, 1+arg*(power(M, nIter-1)-L)) = V((arg-1)*r+1:(arg-1)*r+r, eigOne(1));
    end
    
    for it = 2:nIter
        pow = power(M, it-1);
        for k = 1:pow*R            
            if mod(k, M) ~= 0
                count = 0;
                arg = k - pow/M*count;
                Tmp = zeros(r, 1);
                while arg/(pow/M) > L
                    if arg/(pow/M) < R & count <= R
                        Tmp = Tmp + A{count+1} * Phi(2:r+1, 1+arg*(power(M, nIter-1)/(pow/M)-L));
                    end
                    count = count + 1;
                    arg = k - pow/M*count;
                end
                Phi(2:r+1, 1+k*(power(M, nIter-it)-L)) = Tmp;
            end
        end
    end
    % Offset-Verschiebung berechnen:
    Phi(1, :) = Phi(1, :) + offset;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dimension d = 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif d == 2
    
    % wird spter fr weitere Berechnungen bentigt
    % allZero_A(i, j) == 1 falls A_ij = 0
    %                    0 sonst
    allZero_A = ones(size(A));
    for i = 1:size(A, 1)
        for j = 1:size(A, 2)
            for k = 1:r
                for l = 1:r
                    if A{i, j}(k, l) ~= 0
                        allZero_A(i, j) = 0;
                        break
                    end
                end
                if allZero_A(i, j) == 0
                    break
                end
            end
        end
    end
   
    % Voraussetzung: supp(A) = [0,...,N] hoch 2
    suppA_L = 0;                % kleinster Index
    suppA_R = size(A, 1) - 1;   % grsster Index (=N)
    
    if strcmp(datafile, '') == 1 % d.h. es wurde kein File zum Laden ausgewhlt
                                 % --> EV-Problem lsen!
        % Trger von Phi bestimmen bzw. manuell eingegebenen Trger
        % bernehmen
        if strcmp(manSupp, 'a') % automatisch bestimmen
            disp('Calculating support of scaling vector ...')
            [emsg, suppPhi] = support(A, M);
            %suppPhi
            if strcmp(emsg, '') == 0
                disp('... ERROR!')
                Phi = [];
                errormsg = emsg;
                return
            end
            disp('... OK!')
        else
            suppPhi = manSupp;
        end
            
        % Wissen: suppPhi ist Wuerfel!
        %         Wurde so berechnet, dass das stimmt.
        % suppPhi = [L1,...,R1] x [L2,...,R2]
        L1 = suppPhi(1, 1);
        R1 = suppPhi(1, 3);
        N1 = abs(R1 - L1 + 1);
        L2 = suppPhi(2, 1);
        R2 = suppPhi(2, 3);
        N2 = abs(R2 - L2 + 1);
        % Wissen auch: Phi = 0 auf Rand von suppPhi KEINE GESICHERTE AUSSAGE!!!
        %suppPhi = [L1+1 R1-1 R1-1 L1+1; ...
        %           L2+1 L2+1 R2-1 R2-1]
            
        % Dimension des Eigenwert/-vektor Problems ermitteln
        dimEV = N1 * N2 * r;
        
        disp(['Constructing matrix for eigenvector-problem (matrixdimension = ', ...
              num2str(dimEV), ' x ', num2str(dimEV), ') ...'])
            
        % Matrix zum EV-Problem aufstellen
        % Vorsicht: Es koennen sehr grosse Dimensionen auftreten, aber viele Nullen
        % daher: duennbesetzte Matrix (sparse) verwenden
        
        % Untere Grenze fr Dimension, ab der eine Sparse-Matrix verwendet wird.
        bound_sparse_P = 400;
        % Fehlertoleranz bei der numerischen Bestimmung der EW einer Sparse-Matrix
        % EW nicht unbedingt exakt(!) gleich 1.
        epsilon_EW = 1e-13;
        
        if dimEV < bound_sparse_P
            % Dimension des EV-Problems erlaubt exakte Berechnung aller EWe
            P = zeros(dimEV, dimEV);
        else
            P = sparse(dimEV, dimEV);
        end
        
        % fuer die Fortschrittsanzeige
          finished = 0;
          finished_step = 100 / N1;
        for v1 = L1:R1
            for v2 = L2:R2
                v = [v1; v2];
                % zugehrige Zeile in Matrix P, beginnt bei 0, unten wird noch
                % eins draufaddiert
                row = ( (v1 - L1) * N2 + v2 - L2 ) * r;
                % Teil des unten bentigten Argumentes vorberechnen
                argument0 = M * v;
                for k1 = suppA_L:suppA_R
                    for k2 = suppA_L:suppA_R
                        k = [k1; k2]; % Index des Symbols
                        argument = argument0 - k;
                        if inCube(argument, suppPhi)
                            % "Spalte", in der A_k eingetragen werden muss
                            column = ( (argument(1) - L1) * N2 + argument(2) - L2 ) * r;
                            P(row+1:row+r, column+1:column+r) = A{k1+1, k2+1};
                        end
                    end
                end
            end
            % Fortschrittsanzeige berechnen und ausgeben
            finished = finished + finished_step;
            disp([num2str(round(finished)), ' % finished'])
        end
        
        disp('... OK!')
        
        disp('Solving eigenvector-problem ...')
            
        % Eigenvektoren V und Eigenwerte D bestimmen, es gilt: P*V = V*D
        if dimEV < bound_sparse_P
            [V,D] = eig(P);
        else 
            % hier werden nur die sechs groessten EWe bestimmt ( P ist sparse! )
            [V,D] = eigs(P, 6, 1-1e-15);
        end
        %diag(D)
        % Matrix P wird nicht mehr bentigt:
        clear('P');
        
        disp('... OK!')
        
        % Eigenwert 1 in D suchen
        eigOne = [0, 0];    % eigOne(1) = column, eigOne(2) = number of Eig=1
        for i = 1:size(D, 1)
            % Achtung: D(i,i) == 1 gibt bei approx. EW (wenn P sparse) Probleme!
            if D(i, i) >= 1 - epsilon_EW & D(i, i) <= 1 + epsilon_EW % D(i,i) aus [1-eps_EW, 1+eps_EW]
                eigOne(1) = i;
                eigOne(2) = eigOne(2) + 1;
            end
        end
        %V(:,eigOne(1))
        if eigOne(2) == 0
            errormsg = 'Error: 1 not Eigenvalue of P';
            Phi = [];
            return
        elseif eigOne(2) > 1
            errormsg = 'Error: 1 not unique Eigenvalue of P';
            Phi = [];
            return
        else
            errormsg = '';
        end
        
        % Phi auf ganzzahligem Gitter bestimmen
        old_xy = cell(N1, N2);
        % old_allZero(i, j) = 1 falls (Phi_1, ..., Phi_r)(i,j) = 0
        %                   = 0 sonst
        old_allZero_xy = ones(N1, N2);
        % Gitter initialisieren mit schnitt(supp_Phi, Z hoch 2)
        for k = L1:R1
            for l = L2:R2
                % k und l bestimmen eindeutig Vektor v mit
                % v = k * [1; 0] + l * [0; 1];
                v = [k; l];
                row = ( (k - L1) * N2 + l - L2 ) * r;
                % Funktionswerte eintragen, letzte Zeile =1 bedeutet Wert schon berechnet,
                % nmlich gerade jetzt, =0 heisst, Wert muss im nchsten Iterationsschritt
                % berechnet werden
                old_xy{k-L1+1, l-L2+1} = [v; V(row+1:row+r, eigOne(1)); 1];
                if sum(V(row+1:row+r, eigOne(1))) ~= 0 % d.h. Phi_i ~= 0 fr ein i
                    %suppPhi
                    %v
                    %V(row+1:row+r, eigOne(1))
                    %row
                    old_allZero_xy(k-L1+1, l-L2+1) = 0;
                end
            end
        end
        
        if saving
            global fileName;
            file = ['./Data/Plot/', fileName, '_M', num2str(M(1,1)), '_', num2str(M(1,2)), '_', ...
                    num2str(M(2,1)), '_', num2str(M(2,2)), '_Sx', num2str(L1+offset(1)), '_', num2str(R1+offset(1)), 'y', ...
                    num2str(L2+offset(2)), '_', num2str(R2+offset(2)), '.mat'];
            save(file, 'suppPhi', 'old_xy', 'old_allZero_xy');
            disp(['Data of eigenvector-problem saved in file ''', file, ''''])
        end
        
    else % Daten laden
        load(datafile, 'suppPhi', 'old_xy', 'old_allZero_xy');
        disp(['Loaded data of eigenvector-problem from file ''', datafile, ''''])
        
        % Wissen: suppPhi ist Wuerfel!
        %         Wurde so berechnet, dass das stimmt.
        % suppPhi = [L1,...,R1] x [L2,...,R2]
        L1 = suppPhi(1, 1);
        R1 = suppPhi(1, 3);
        N1 = abs(R1 - L1 + 1);
        L2 = suppPhi(2, 1);
        R2 = suppPhi(2, 3);
        N2 = abs(R2 - L2 + 1);
        
        errormsg = ''; % Kein Fehler zu erwarten
    end
    
    old_bound_k = L1;
    old_bound_l = L2;
    old_mj      = 1;
    
    m = abs(det(M));
    disp('Starting iteration ...')
    for j = 1:nIter
        disp(['... ', num2str(j), ' of ', num2str(nIter)])
        MM = mpower(M, j);
        if mod(j, 2) == 1   % d == 2
            mj = power(m, -1 * (j + 1)/2);
            s = [1.0 * mj ; 0]; % = 1/mj * [1;0]
            t = [0; 1.0 * mj];  % = 1/mj * [0;1]
            TT = mpower([s t], -1);
            l1 = TT * [L1; L2];
            l2 = TT * [R1; R2];
                        
            % nur ganzzahlige Koeffizienten erwnscht
            l1 = ceil(l1);
            l2 = floor(l2);
            
            bounds_k = [l1(1) l2(1)];
            bounds_l = [l1(2) l2(2)];
            
            xy = cell(bounds_k(2)-bounds_k(1)+1, bounds_l(2)-bounds_l(1)+1);
            allZero_xy = ones(bounds_k(2)-bounds_k(1)+1, bounds_l(2)-bounds_l(1)+1);
            
            for k = bounds_k(1):bounds_k(2)
                for l = bounds_l(1):bounds_l(2)
                    v = k * [1; 0] + l * [0; 1];
                    mjv = mj * v;
                    
                    % mjv evtl. schon berechnet? Wenn ja, Wert bernehmen
                    ind1 = mjv(1)/old_mj - old_bound_k + 1;
                    ind2 = mjv(2)/old_mj - old_bound_l + 1;
                    index = [ind1; ind2];
                    if index == floor(index)
                        %if index >= [1; 1] & index <= [size(xy, 1); size(xy, 2)]
                            %if xy{ind1, ind2}(1:2, 1) == v_real
                            xy{k-bounds_k(1)+1, l-bounds_l(1)+1} = [mjv; old_xy{ind1, ind2}(3:end-1, 1); 2];
                            allZero_xy(k-bounds_k(1)+1, l-bounds_l(1)+1) = old_allZero_xy(ind1, ind2);
                            %end
                            continue
                            %end
                    end
                                        
                    %if inCube(mjv, suppPhi) %msste eigentlich immer der Fall sein, count kontrollieren!!!
                        tmp = MM * mjv;
                        if tmp == floor(tmp) % d.h. tmp ist in Z^d, d.h. Wert von Phi muss jetzt bestimmt werden
                            val = calcPhi(mjv, old_xy, old_mj, M, old_bound_k, old_bound_l, suppA_L, suppA_R, suppPhi, A, allZero_A, old_allZero_xy);
                            xy{k-bounds_k(1)+1, l-bounds_l(1)+1} = [mjv; val; 1];
                            if sum(val) ~= 0
                                allZero_xy(k-bounds_k(1)+1, l-bounds_l(1)+1) = 0;
                            end
                        else
                            xy{k-bounds_k(1)+1, l-bounds_l(1)+1} = [mjv; zeros(r, 1); 0];
                        end
                        %end
                end
            end
        else
            mj = power(m, -1 * j/2);
            for k = 1:size(xy, 1)
                for l = 1:size(xy, 2)
                    if xy{k, l}(end, 1) == 0
                        v = xy{k, l}(1:2, 1);
                        val = calcPhi(v, xy, mj, M, bounds_k(1), bounds_l(1), suppA_L, suppA_R, suppPhi, A, allZero_A, allZero_xy);
                        xy{k, l} = [v; val; 1];
                        if sum(val) ~= 0
                            allZero_xy(k, l) = 0;
                        end
                    end
                end
            end
            old_xy = xy;
            old_bound_k = bounds_k(1);
            old_bound_l = bounds_l(1);
            old_mj      = mj;
            old_allZero_xy = allZero_xy;
        end
     
    end
       
    % bentigtes Format der Werte von Phi erzeugen
    Phi = [];
    if mod(nIter, 2) == 0
        for i = 1:size(xy, 1)
            for j = 1:size(xy, 2)
                Phi = [Phi xy{i,j}(1:end-1, 1)];
            end
        end
    else
        for i = 1:size(old_xy, 1)
            for j = 1:size(old_xy, 2)
                Phi = [Phi old_xy{i,j}(1:end-1, 1)];
            end
        end
        for i = 1:size(xy, 1)
            for j = 1:size(xy, 2)
                if xy{i, j}(end, 1) == 1    % hier kommen Werte hinzu, die schon in old_xy
                                            % enthalten sind! Fr Triangulierung unerheblich! Jetzt nicht mehr!?
                    Phi = [Phi xy{i,j}(1:end-1, 1)];
                end
            end
        end
    end
    % Phi verschieben
    Phi(1, :) = Phi(1, :) + ones(1, size(Phi, 2)) * offset(1);
    Phi(2, :) = Phi(2, :) + ones(1, size(Phi, 2)) * offset(2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dimension d > 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
    Phi = [];
    errormsg = 'Function only works for d = 1 and d = 2!';
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function res = calcPhi(v_real, xy, mj, M, bound_k, bound_l, suppA_L, suppA_R, suppPhi, A, allZero_A, allZero_xy)
% v_real            : Koordinaten, an denen Phi berechnet wird
% xy                : Gitter fuer Berechnung
% mj                : |detM|^iter
% M                 : Dilatationsmatrix
% bound_k, bound_l  : linke, untere Ecke des Trgers von Phi
% suppA_L, suppA_R  : Grenzen von suppA
% suppPhi           : Traeger von Phi
% A                 : Maske
% allZero_A         : Information ber Maske
% allZero_xy        : Information ber Gitter
%
% Berechnet Phi an der Stelle v_real.

res = zeros(size(xy{1, 1}, 1) - 3, 1);
for k1 = suppA_L:suppA_R
    for k2 = suppA_L:suppA_R
        if allZero_A(k1+1, k2+1) == 0
            argument = M * v_real - [k1; k2];
            if inCube(argument, suppPhi)
                k_arg = argument(1)/mj - bound_k + 1;
                l_arg = argument(2)/mj - bound_l + 1;
                if ~allZero_xy(k_arg, l_arg)
                    res = res + A{k1 + 1, k2 + 1} * xy{k_arg, l_arg}(3:end-1, 1);
                end
            end
        end
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [errormsg, supp] = support(A, M)
% A     : Maske im internen Format
% M     : Dilatationsmatrix
%
% Berechnet den Trger eines Skalierungsvektors.

% Dimension
d = size(M, 1);
% Multiplizitt
r = size(A{1}, 1);

if d > 2 | d < 2
    errormsg = 'Error: support only works for d = 2 !';
    supp = [];
    return
end

if det(M) == 0
    errormsg = 'Error: M not invertible !';
    supp = [];
    return
end

N = size(A, 1) - 1;

% Bestimme l, s.d. M hoch l = m * I, m aus Z
nIter = 10;
l = 0;
if not(M(1,2) == 0 & M(2,1) == 0 & M(1,1) == M(2,2))
    for k = 2:nIter
        powM = mpower(M, k);
        if powM(1,2) == 0 & powM(2,1) == 0 & powM(1,1) == powM(2,2)
            l = k;
            break
        end
    end
else
    l = 1;
end
%l
if l == 0
    errormsg = ['Error: M^l != k*I for 1 <= l <= ', num2str(nIter), ', k in Z !'];
    supp = [];
    return
else
    errormsg = '';
end

% konvexe Huelle von supp(A) bestimmen
% ist in unserem Fall gleich supp(A)!!!!!!!!!!!!!!!!!!!!!!!!!

% supp(A) bestimmen
suppA = [0 N N 0; ...
         0 0 N N];

%supp(Phi) berechnen
supp = suppA;
if l > 1
    supp = minkowskiSum(supp, M * suppA, det(M)); % k = 1
    for k = 2:l-1
        powM = mpower(M, k);
        supp = minkowskiSum(supp, powM * suppA, det(powM));
    end
end
% kleinsten Wrfel, der supp enthlt, bestimmen:
supp = findCube(supp);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function phi = angle(u, v)
% u     : stehender Vektor
% v     : stehender Vektor
%
% Liefert den Winkel, um den man Vektor v gegen den Uhrzeigersinn
% bis auf v drehen muss.

deter      = det([u, v]);
norm_u     = norm(u);
norm_v     = norm(v);
if norm_u == 0 | norm_v == 0
    phi = 0;
else
    kosPhi_rad = dot(u, v)/(norm(u)*norm(v));
    phi_rad    = acos(kosPhi_rad);
    phi        = phi_rad * 180 / pi;
    if deter < 0
        phi = 360 - phi;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function poly_sorted = polySort(poly_unsorted, reverse)
% poly_unsorted  : Polygon, Spalten bestimmen Eckpunkte,
%                  nicht geschlossen
% reverse        : < 1 bedeutet, Polygonpunkte sind im
%                  Uhrzeigersinn angeben -> ndern in
%                  Nummerierung gegen den Uhrzeigersinn,
%                  sonst keine nderung notwendig
%
% ndert die Eckpunktreihenfolge, s.d. erster Punkt in 
% Matrix bzgl. y-Koordinate minimal ist. Falls nicht 
% eindeutig, wird zustzlich nach minimalem x sortiert.

n = size(poly_unsorted, 2);
if reverse < 1 % Nummerierung rechts rum -> aendern
    poly_unsorted = poly_unsorted(:, n:-1:1);
end
index_min = 1;
for i = 2:n
    if poly_unsorted(2, i) < poly_unsorted(2, index_min)
        index_min = i;
    elseif poly_unsorted(2, i) == poly_unsorted(2, index_min)
        if poly_unsorted(1, i) < poly_unsorted(1, index_min)
            index_min = i;
        end
    end
end

if index_min == 1
    poly_sorted = poly_unsorted;
else
    poly_sorted = [poly_unsorted(:, index_min:n) poly_unsorted(:, 1:index_min-1)];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sumPR = minkowskiSum(P, R, detPowM)
% P,R : Matrizen, deren Spalten jeweils die Ecken eines 
%       nicht geschlossenen, konvexen Polygons darstellen.
%       Angabe der Ecken gegen den Uhrzeigersinn!
%       P = {v1, ..., vn}
%       R = {w1, ..., wm}
% detPowM : Determinante der Potenzierten Matrix M,
%           < 1 beduetet, Angabe Eckpunkte von Polygon R 
%           im Uhrzeigersinn -> macht Umkehrung notwendig
%
% Liefert die Minkowski Summe P + R zweier konvexer Polygone.
% Beachte: P(:,1) und R(:,1) muessen y-minimal sein!!!
%   (wird automatisch sortiert!)
% P + R = {x + y | x in P, y in R}

n = size(P, 2);
m = size(R, 2);
% Polygone sortieren, s.d. 
P = polySort(P, 1);
R = polySort(R, detPowM);

% Polygone "schliessen":
P = [P P(:, 1)];
R = [R R(:, 1)];

% Summe berechnen:
i = 1;
j = 1;

sumPR = []; % Summe von P und R
while i ~= n+1 | j ~= m+1
    % v_i und v_i+1 bestimmen:
    vi  = P(:, i);
    if i == n+1
        vi1 = P(:, 2);
    else
        vi1 = P(:, i+1);
    end
    % Winkel von Vektor v_i+1-v_i zur positiven x-Achse
    % bestimmen:
    angle_vi_vi1 = angle([1;0], vi1 - vi);
    
    % w_j und w_j+1 bestimmen:
    wj  = R(:, j);
    if j == m+1
        wj1 = R(:, 2);
    else
        wj1 = R(:, j+1);
    end
    % Winkel von Vektor w_j+1-w_j zur positiven x-Achse
    % bestimmen:
    angle_wj_wj1 = angle([1;0], wj1 - wj);
    
    % Summe von v_i und w_j als Ecke von P+R eintragen
    sumPR = [sumPR (vi + wj)];
    % nchsten Eckpunkte bestimmen:
    if angle_vi_vi1 < angle_wj_wj1
        i = i + 1;
    elseif angle_vi_vi1 > angle_wj_wj1
        j = j + 1;
    else %if angle_vi_vi1 == angle_wj_wj1
        i = i + 1;
        j = j + 1;
    end
    if i > n+1
        i = n + 1;
        if j <= m
            j = j + 1;
        end
    end
    if j > m+1
        j = m + 1;
        if i <= n
            i = i + 1;
        end
    end
end

% NUR ZUM TESTEN:
% sum = sumPR;
% sum = [sum sum(:,1)];
% figure
% hold on;
% %figure
% line(P(1,:),P(2,:),'LineWidth',4,'Color','r')
% %figure
% line(R(1,:),R(2,:),'LineWidth',4,'Color','g')
% %figure
% line(sum(1,:),sum(2,:),'LineWidth',4,'Color','b')

% figure
% hold on;
% fill(P(1,:),P(2,:),'r')
% fill(R(1,:),R(2,:),'g')
% fill(sumPR(1,:),sumPR(2,:),'b')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function cube = findCube(poly)
% poly     : konvexes Polygon
%
% Liefert den kleinsten Wrfel, in dem das Polygon poly
% enthalten ist.

minX = min(poly(1, :));
maxX = max(poly(1, :));
minY = min(poly(2, :));
maxY = max(poly(2, :));
cube = [minX maxX maxX minX; ...
        minY minY maxY maxY];

% NUR ZUM TESTEN:
% line([cube(1,:) cube(1,1)], [cube(2,:) cube(2,1)],'LineWidth',4,'Color','y')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function in = inCube(p, cube)
% p        : 2-dimensionaler Vektor
% cube     : 2-dimensionaler Wrfel
%
% Prft, ob p innerhalb von cube liegt.

n = size(cube, 2);
minX = cube(1, 1);
maxX = cube(1, n-1);
minY = cube(2, 1);
maxY = cube(2, n-1);
x = p(1);
y = p(2);
if x >= minX & x <= maxX & y >= minY & y <= maxY
    in = 1;
else
    in = 0;
end
